
#####

make_asr_table <- function (tree,tip_values,node_values,youngest_tip) {

# youngest_tip must be 0 (=extant) OR a negative number (e.g., -10 is 10mya)

##################################################

# check the input arguments

message_1="'youngest_tip' must be either: 0 (if there are extant taxa at tips) OR a negative number (e.g., -15 is 15mya)."
if(youngest_tip>0) stop(message_1)

message_2="'tree' must be of class 'phylo'."
tree_test=class(tree)
if(tree_test!="phylo") stop(message_2)

message_3="'tip_values' must be of class 'matrix'. The matrix must contain two columns where column-1 are tip-numbers & column-2 are trait-values."
tip_test=class(tip_values)
tip_test=tip_test[1]
if(tip_test!="matrix") stop(message_3)

message_4="'node_values' must be of class 'matrix'. The matrix must contain two columns where column-1 are node-numbers & column-2 are trait-values."
node_test=class(node_values)
node_test=node_test[1]
if(node_test!="matrix") stop(message_4)

message_5="There is a different number of tips in 'tree' and 'tip_values'. These must match."
n_tips_in_tree=length(tree$tip.label)
n_tips_in_table=dim(tip_values)[1]
if(n_tips_in_tree!=n_tips_in_table) stop(message_5)

message_6="There is a different number of internal nodes in 'tree' and 'node_values'. These must match."
n_nodes_in_tree=tree$Nnode
n_nodes_in_table=dim(node_values)[1]
if(n_nodes_in_tree!=n_nodes_in_table) stop(message_6)

##################################################

# build the asr table columns 1-4

data_matrix=tree$edge
n_edges=dim(data_matrix)[1]
data_matrix=cbind(data_matrix,data_matrix)
my_seq=seq(from=1,to=n_edges,by=1)
data_matrix[,1]=my_seq
edge_lengths=tree$edge.length
data_matrix[,2]=edge_lengths
colnames(data_matrix)=c("Edge_number","Edge_length","Parent_node","Child_node")
#data_matrix

#####

# add empty columns 5-8

data_matrix=cbind(data_matrix,data_matrix)
data_matrix[,5:8]=NA
a=c("Edge_number","Edge_length","Parent_node","Child_node")
b=c("Parent_height","Child_height","Parent_trait_value","Child_trait_value")
ab=c(a,b)
colnames(data_matrix)=ab
#data_matrix

#####

# populate parent heights

count=1
repeat {
temp_node=data_matrix[count,3]
data_matrix[count,5]=nodeheight(tree,temp_node)
count=count+1
if(count==n_edges+1) break }
#data_matrix

#####

# populate child heights

count=1
repeat {
temp_node=data_matrix[count,4]
data_matrix[count,6]=nodeheight(tree,temp_node)
count=count+1
if(count==n_edges+1) break }
#data_matrix

#####

# populate parent trait values

count=1
repeat {
temp_node=data_matrix[count,3]
matched_node=which(node_values[,1]==temp_node)
temp_value=node_values[matched_node,2]
data_matrix[count,7]=temp_value
count=count+1
if(count==n_edges+1) break }
#data_matrix

#####

# populate child trait values

n_tips=length(tree$tip.label)
root_node=n_tips+1

count=1
repeat {
temp_node=data_matrix[count,4]

if(temp_node<root_node) {
matched_node=which(tip_values[,1]==temp_node)
temp_value=tip_values[matched_node,2]
data_matrix[count,8]=temp_value }

if(temp_node>root_node) {
matched_node=which(node_values[,1]==temp_node)
temp_value=node_values[matched_node,2]
data_matrix[count,8]=temp_value }

count=count+1
if(count==n_edges+1) break }
#data_matrix

#####

# adjust node heights to node ages

max_tree_height=max(data_matrix[,6])
data_matrix[,6]=data_matrix[,6]-max_tree_height
data_matrix[,5]=data_matrix[,5]-max_tree_height
data_matrix[,6]=data_matrix[,6]+youngest_tip
data_matrix[,5]=data_matrix[,5]+youngest_tip
a=c("Edge_number","Edge_length","Parent_node","Child_node")
b=c("Parent_age","Child_age","Parent_trait_value","Child_trait_value")
ab=c(a,b)
colnames(data_matrix)=ab
#data_matrix

##################################################

return(data_matrix) }


