
#####

interpolate_asr_table <- function(asr_table,increment) {

# chosen increment must be a positive number (e.g., 1 is 1my time-slices)
# chosen increment must produce at least 10 time-slices

##################################################

# check the input arguments

message_1="'increment' must be a positive number (e.g., 1 or 0.5 are time-slices in increments of 1 and 0.5 (?my), respectively). Negative numbers are not allowed. Zero is not allowed."
if(increment<=0) stop(message_1)

message_2a="The chosen value for 'increment' will produce less than 10 time-slices. Please choose a smaller increment. "
message_2b="Smaller values produce better resolution -- you can always filter out 'too many' time-slices."
oldest_age=min(asr_table[,5])
youngest_age=max(asr_table[,6])
age_range=youngest_age-oldest_age
max_increment=age_range/10
if(increment>max_increment) stop(message_2a,message_2b)

##################################################

# determine slice ages to interpolate

increment=round(increment,digits=2)

oldest_age=min(asr_table[,5])
youngest_age=max(asr_table[,6])
#oldest_age
#youngest_age

youngest_slice=youngest_age+(2*increment)
youngest_slice=round(youngest_slice,digits=0)
if(youngest_slice>0) { youngest_slice=0 }
oldest_slice=oldest_age-(2*increment)
oldest_slice=round(oldest_slice,digits=0)
#youngest_slice
#oldest_slice

slices=seq(from=oldest_slice,to=youngest_slice,by=increment)
slices=round(slices,digits=2)

#####

# setup empty interpolation table

n_slices=length(slices)
n_edges=dim(asr_table)[1]

interpolation_table=matrix(,nrow=n_edges,ncol=n_slices)
rownames(interpolation_table)=asr_table[,1]
colnames(interpolation_table)=slices

#####

# populate the interpolation table

current_edge_number=1
repeat {

slices_a=which(slices>=asr_table[current_edge_number,5])
slices_a=slices[slices_a]
slices_b=which(slices_a<=asr_table[current_edge_number,6])
slices_b=slices_a[slices_b]

slices_to_estimate=slices_b
n_slices_to_estimate=length(slices_to_estimate)

if(n_slices_to_estimate>0) { ###from_here
value_change=asr_table[current_edge_number,8]-asr_table[current_edge_number,7]
time_change=asr_table[current_edge_number,2]
edge_rate=value_change/time_change

slice_count=1
repeat {
current_slice=slices_to_estimate[slice_count]
time_since_start=current_slice-asr_table[current_edge_number,5]
start_value=asr_table[current_edge_number,7]
interpolated_value=start_value+(time_since_start*edge_rate)
row_num=which(rownames(interpolation_table)==current_edge_number)
col_num=which(colnames(interpolation_table)==current_slice)
interpolation_table[row_num,col_num]=interpolated_value
slice_count=slice_count+1
if(slice_count==n_slices_to_estimate+1) break }
} ###to_here

current_edge_number=current_edge_number+1
if(current_edge_number==n_edges+1) break }

#####

# remove columns with all NAs

NA_test_a=colSums(interpolation_table,na.rm=TRUE)
NA_test_b=which(NA_test_a==0)
if(length(NA_test_b)>0) { interpolation_table=interpolation_table[,-NA_test_b] }

##################################################

return(interpolation_table) }


